import java.io.IOException;
import java.security.GeneralSecurityException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;

public class Main {
    public static void main(String[] args) {
        InputParser parser = new FileInputParser();
        InterestLinker linker = new UserInterests();
        ArrayList<User> users = new ArrayList<>();

        try {
            HashMap<String, List<String>> namesAndHistory = parser.parseUsersAndHistory(Main.class.getResource("input.txt").getPath());

            for (String name : namesAndHistory.keySet()) {
                List<String> history = namesAndHistory.get(name);
                List<String> interests = linker.linkHistoryToInterests(history);
                users.add(new User(name, history, interests));
            }
        } catch (IOException e) {
            e.printStackTrace();
        }

        ArrayList<List<VideoSuggestion>> suggestions = new ArrayList<>(users.size());

        try {
            YouTubeSearcher yt = new YouTubeSearcher();

            for (User user : users) {
                suggestions.add(yt.findSuggestions(user.interests));
            }
        } catch (GeneralSecurityException | IOException e) {
            e.printStackTrace();
        }

        FileOutputWriter outputWriter = new FileOutputWriter();
        outputWriter.writeUsersAndSuggestions(users, suggestions);
    }
}
