import com.google.api.client.googleapis.javanet.GoogleNetHttpTransport;
import com.google.api.client.http.javanet.NetHttpTransport;
import com.google.api.client.json.JsonFactory;
import com.google.api.client.json.jackson2.JacksonFactory;
import com.google.api.services.youtube.YouTube;
import com.google.api.services.youtube.model.SearchListResponse;
import com.google.api.services.youtube.model.SearchResult;

import java.io.IOException;
import java.security.GeneralSecurityException;
import java.util.ArrayList;
import java.util.List;

public class YouTubeSearcher implements VideoFinder {
    private static final String APPLICATION_NAME = "API code samples";
    private static final JsonFactory JSON_FACTORY = JacksonFactory.getDefaultInstance();

    private YouTube youtubeService;

    /**
     * Build and return an API client service.
     *
     * @return an authorized API client service
     * @throws GeneralSecurityException, IOException
     */
    YouTubeSearcher() throws GeneralSecurityException, IOException {
        final NetHttpTransport httpTransport = GoogleNetHttpTransport.newTrustedTransport();
        youtubeService = new YouTube.Builder(httpTransport, JSON_FACTORY, null)
                .setApplicationName(APPLICATION_NAME)
                .build();
    }

    /**
     * Call function to create API service object. Define and
     * execute API request. Print API response.
     *
     * @throws IOException, GoogleJsonResponseException
     */
    @Override
    public List<VideoSuggestion> findSuggestions(List<String> tags) throws IOException {
        YouTube.Search.List request = youtubeService.search()
                .list("snippet")
                .setKey("AIzaSyCFjrFjnH1lHZb6hKjP8N7tlKsRvyC-3oI")
                .setOrder("relevance");

        List<SearchResult> results;

        if (tags.size() < 5) {
            request.setMaxResults(5L);
            request.setQ(String.join(" ", tags));
            SearchListResponse response = request.execute();
            results = response.getItems();
        } else {
            results = new ArrayList<>();

            for (String tag : tags) {
                request.setMaxResults(1L);
                request.setQ(tag);
                SearchListResponse response = request.execute();
                results.addAll(response.getItems());
            }
        }

        ArrayList<VideoSuggestion> suggestions = new ArrayList<>(results.size());

        for (SearchResult result : results) {
            if (result.getId().getVideoId() != null) {
                suggestions.add(new VideoSuggestion(result));
            }
        }
        return suggestions;
    }
}